@@ -47,6 +47,7 @@ import Data.Distributive
 import Data.Foldable as Foldable
 import Data.Functor.Bind
 import Data.Functor.Classes
+import Data.Functor.Product
 import Data.Functor.Rep as Rep
 import Data.HashMap.Lazy (HashMap)
 import Data.Hashable
@@ -56,6 +57,12 @@ import Data.Hashable.Lifted
 import Data.IntMap (IntMap)
 import Data.Ix
 import Data.Map (Map)
+#if !(MIN_VERSION_base(4,8,0))
+import Data.Monoid (Monoid(..))
+#endif
+#if !(MIN_VERSION_base(4,11,0))
+import Data.Semigroup (Semigroup)
+#endif
 import Data.Serialize as Cereal
 import Data.Vector (Vector)
 import qualified Data.Vector.Generic.Mutable as M
@@ -79,6 +86,7 @@ import Linear.V2
 import Linear.V3
 import Linear.V4
 import Linear.Vector
+import System.Random
 
 #ifdef HLINT
 {-# ANN module "HLint: ignore Unused LANGUAGE pragma" #-}
@@ -107,6 +115,12 @@ class Additive (Diff p) => Affine p where
   p .-^ v = p .+^ negated v
   {-# INLINE (.-^) #-}
 
+instance (Affine f, Affine g) => Affine (Product f g) where
+  type Diff (Product f g) = Product (Diff f) (Diff g)
+  Pair a b .-. Pair c d = Pair (a .-. c) (b .-. d)
+  Pair a b .+^ Pair c d = Pair (a .+^ c) (b .+^ d)
+  Pair a b .-^ Pair c d = Pair (a .+^ c) (b .+^ d)
+
 -- | Compute the quadrance of the difference (the square of the distance)
 qdA :: (Affine p, Foldable (Diff p), Num a) => p a -> p a -> a
 qdA a b = Foldable.sum (fmap (join (*)) (a .-. b))
@@ -148,7 +162,8 @@ newtype Point f a = P (f a)
            , Eq1, Ord1, Show1, Read1
            , Traversable, Apply, Additive, Metric
            , Fractional , Num, Ix, Storable, Epsilon
-           , Hashable
+           , Semigroup, Monoid
+           , Random, Hashable
 #if __GLASGOW_HASKELL__ >= 702
            , Generic
 #endif
