@@ -30,6 +30,9 @@
 #define MIN_VERSION_transformers(x,y,z) 1
 #endif
 
+#ifndef MIN_VERSION_base
+#define MIN_VERSION_base(x,y,z) 1
+#endif
 
 -----------------------------------------------------------------------------
 -- |
@@ -90,6 +93,7 @@ import Linear.Vector
 #if __GLASGOW_HASKELL__ >= 707
 import Linear.V
 #endif
+import System.Random
 import Prelude hiding (sum)
 
 -- $setup
@@ -122,6 +126,12 @@ instance Finite V0 where
   fromV _ = V0
 #endif
 
+instance Random (V0 a) where
+  random g = (V0, g)
+  randomR _ g = (V0, g)
+  randomRs _ _ = repeat V0
+  randoms _ = repeat V0
+
 instance Serial1 V0 where
   serializeWith _ = serialize
   deserializeWith _ = deserialize
@@ -166,6 +176,15 @@ instance Applicative V0 where
   V0 <*> V0 = V0
   {-# INLINE (<*>) #-}
 
+instance Semigroup (V0 a) where
+  _ <> _ = V0
+
+instance Monoid (V0 a) where
+  mempty = V0
+#if !(MIN_VERSION_base(4,11,0))
+  mappend _ _ = V0
+#endif
+
 instance Additive V0 where
   zero = V0
   {-# INLINE zero #-}
