@@ -29,6 +29,9 @@
 #define MIN_VERSION_transformers(x,y,z) 1
 #endif
 
+#ifndef MIN_VERSION_base
+#define MIN_VERSION_base(x,y,z) 1
+#endif
 
 -----------------------------------------------------------------------------
 -- |
@@ -49,6 +52,7 @@ module Linear.V2
   , ex, ey
   , perp
   , angle
+  , unangle
   , crossZ
   ) where
 
@@ -99,6 +103,7 @@ import Linear.V
 import Linear.Vector
 import Linear.V1 (R1(..),ex)
 import Prelude hiding (sum)
+import System.Random
 
 -- $setup
 -- >>> import Control.Lens
@@ -137,6 +142,16 @@ instance Finite V2 where
   fromV (V v) = V2 (v V.! 0) (v V.! 1)
 #endif
 
+instance Random a => Random (V2 a) where
+  random g = case random g of
+   (a, g') -> case random g' of
+     (b, g'') -> (V2 a b, g'')
+  {-# inline random #-}
+  randomR (V2 a b, V2 c d) g = case randomR (a, c) g of
+    (x, g') -> case randomR (b, d) g' of
+      (y, g'') -> (V2 x y, g'')
+  {-# inline randomR #-}
+
 instance Functor V2 where
   fmap f (V2 a b) = V2 (f a) (f b)
   {-# INLINE fmap #-}
@@ -434,6 +449,13 @@ instance MonadFix V2 where
 angle :: Floating a => a -> V2 a
 angle a = V2 (cos a) (sin a)
 
+unangle :: (Floating a, Ord a) => V2 a -> a
+unangle a@(V2 ax ay) =
+  let alpha = asin $ ay / norm a
+  in if ax < 0
+       then pi - alpha
+       else alpha
+
 -- | The Z-component of the cross product of two vectors in the XY-plane.
 --
 -- >>> crossZ (V2 1 0) (V2 0 1)
@@ -488,3 +510,12 @@ instance Field1 (V2 a) (V2 a) a a where
 
 instance Field2 (V2 a) (V2 a) a a where
   _2 f (V2 x y) = f y <&> \y' -> V2 x y'
+
+instance Semigroup a => Semigroup (V2 a) where
+ (<>) = liftA2 (<>)
+
+instance Monoid a => Monoid (V2 a) where
+  mempty = pure mempty
+#if !(MIN_VERSION_base(4,11,0))
+  mappend = liftA2 mappend
+#endif
