@@ -1,5 +1,9 @@
+{-# LANGUAGE CPP #-}
 module Binary (tests) where
+
+#if !(MIN_VERSION_base(4,8,0))
 import Control.Applicative
+#endif
 import Data.Binary.Put
 import Data.Binary.Get
 import Linear
